﻿using Microsoft.Owin;

[assembly: OwinStartupAttribute(typeof(CelloSaaSWebClient.Startup))]
namespace CelloSaaSWebClient
{
    using CelloSaaS.Library;
    using CelloSaaS.Owin.Core.Analytics.Middlewares;
    using CelloSaaS.Owin.Security.GenericOAuth2Authentication;
    using CelloSaaS.Owin.Security.GenericOAuth2Authentication.Provider;
    using CelloSaaS.Security.Extensions;
    using CelloSaaS.Security.Options.BearerAuthentication;
    using CelloSaaS.Security.Options.CookieAuthentication;
    using CelloSaaS.ServiceContracts.SettingsManagement;
    using CelloSaaS.ServiceProxies.TenantManagement;
    using CelloSaaS.View;
    using Microsoft.AspNet.Identity;
    using Microsoft.Owin;
    using Microsoft.Owin.Cors;
    using Microsoft.Owin.Security.Cookies;
    using Microsoft.Owin.Security.OAuth;
    using Owin;
    using System.Collections.Generic;
    using System.IdentityModel.Tokens;
    using System.Linq;
    using System.Security.Claims;
    using System.Threading.Tasks;
    using WebApplication.Models;
    using System;
    using CelloSaaS.Library.Helpers;
    using System.Text.RegularExpressions;

    /// <summary>
    /// The StartUp Class
    /// </summary>
    public class Startup
    {
        static string FileNamesPattern = ConfigHelper.AllowableFileExtensions();
        Func<Uri, bool> isFile = (reqUri) =>
        {
            return !(reqUri.IsFile ||
            (reqUri.PathAndQuery.ToLowerInvariant().StartsWith("/Account/FederatedResult") || reqUri.PathAndQuery.ToLowerInvariant().StartsWith("/bundles/")) ||
            Regex.IsMatch(reqUri.PathAndQuery, FileNamesPattern, RegexOptions.IgnoreCase));
        };

        /// <summary>
        /// The Configuration for App Initialization
        /// </summary>
        /// <param name="app"></param>
        public void Configuration(IAppBuilder app)
        {
            app.UseCookieAuthentication(new CookieAuthenticationOptions
            {
                AuthenticationType = DefaultAuthenticationTypes.ApplicationCookie,
                CookieName = "FormAuthentication",
                LoginPath = new PathString("/Account/Logon"),
                SlidingExpiration = true,
            })
            .UseExternalSignInCookie(DefaultAuthenticationTypes.ExternalCookie);

            app.UseCors(CorsOptions.AllowAll);

            //app.MapWhen(ctx => isFile(ctx.Request.Uri), analyticsApp =>
            //{
            //    analyticsApp.UseCelloUserActivity(new CelloSaaS.Owin.Core.Analytics.Middlewares.Options.UserActivityOptions
            //    {
            //        IsEnabled = true
            //    }).
            //    UseProductAnalytics(new CelloSaaS.Owin.Core.Analytics.Middlewares.Options.ProductAnalyticsOptions
            //    {
            //        IsEnabled = true
            //    });
            //});

            CelloAuthServerRegistration(app);

            // Implicit flows Middleware registrations
            ApiAuthentication(app);

            CelloAuthServerRegistration(app);
        }

        private void ApiAuthentication(IAppBuilder app)
        {
            var jwtAuthOptions = new Microsoft.Owin.Security.Jwt.JwtBearerAuthenticationOptions();

            jwtAuthOptions.Provider = new OAuthBearerAuthenticationProvider
            {
                OnValidateIdentity = async (ctx) =>
                {
                    var claims = ctx.Ticket.Identity.Claims;
                    await TransformCelloClaims(ctx, claims);
                }
            };

            jwtAuthOptions.TokenValidationParameters = new TokenValidationParameters
            {
                ValidIssuer = System.Configuration.ConfigurationManager.AppSettings["AuthIssuer"],
                ValidAudience = string.Format(System.Globalization.CultureInfo.InvariantCulture, "{0}~{1}~{2}", "2af3d511-7c80-453a-8062-01ec8a7b2ba1", "Web", "MultiTenantAccess"),
                IssuerSigningToken = new X509SecurityToken(CertificateStore.GetCertificate())
            };

            app.MapWhen(req => req.Request.Headers.ContainsKey("Authorization"), apiAuth =>
            {
                apiAuth.UseCelloBasicAuthentication(new CelloSaaS.Security.Options.BasicAuthentication.CelloBasicAuthenticationOptions
                {
                    AuthenticationType = DefaultAuthenticationTypes.ApplicationCookie
                });
                apiAuth.UseJwtBearerAuthentication(jwtAuthOptions);
                apiAuth.UseCelloBearerAuthentication(new CelloBearerAuthenticationOptions());

                apiAuth.UseCelloCookieAuthentication(new CelloCookieAuthenticationOptions());
            });
        }

        private static Task TransformCelloClaims(OAuthValidateIdentityContext ctx, IEnumerable<Claim> claims)
        {
            if (claims == null || claims.Count() < 1) { return Task.FromResult<object>(null); }

            string userId = ctx.Ticket.Identity.FindFirstValue("sub");
            if (string.IsNullOrEmpty(userId))
            {
                userId = ctx.Ticket.Identity.FindFirstValue(ClaimTypes.NameIdentifier);
            }

            if (!string.IsNullOrEmpty(userId))
            {
                ctx.Ticket.Identity.AddClaim(new Claim(CelloClaimTypes.UserId, userId));
            }

            var tenantCode = ctx.Ticket.Identity.FindFirstValue("company_code");

            if (!string.IsNullOrEmpty(tenantCode))
            {
                //set the tenant id claim
                var tenantId = TenantProxy.GetTenantIdFromTenantCode(tenantCode);
                ctx.Ticket.Identity.AddClaim(new Claim(CelloClaimTypes.TenantId, tenantId));
            }

            var userName = ctx.Ticket.Identity.FindFirstValue("name");

            if (string.IsNullOrEmpty(userName))
            {
                ctx.Ticket.Identity.AddClaim(new Claim(ClaimTypes.Name, userName));
            }

            string clientId = ctx.Ticket.Identity.FindFirstValue("client_id");

            if (!string.IsNullOrEmpty(clientId))
            {
                ctx.Ticket.Identity.AddClaim(new Claim(CelloClaimTypes.ClientId, clientId));
            }

            string role = ctx.Ticket.Identity.FindFirstValue("role");
            if (!string.IsNullOrEmpty(role))
                ctx.Ticket.Identity.AddClaim(new Claim(ClaimTypes.Role, role));

            return Task.FromResult<object>(null);
        }

        private static void CelloAuthServerRegistration(IAppBuilder app)
        {
            var authOptions = new GenericOAuth2AuthenticationOptions(AuthenticationConstants.CelloAuthentication)
            {
                SignInAsAuthenticationType = DefaultAuthenticationTypes.ExternalCookie,
                AccessType = AuthenticationConstants.CelloAuthentication,
                AuthenticationMode = Microsoft.Owin.Security.AuthenticationMode.Passive,
                AuthorizationEndpointUri = AppSettingHelper.GetAuthServerUri() + "authorize",
                TokenEndpointUri = AppSettingHelper.GetAuthServerUri() + "api/token",
                UserInfoEndpointUri = AppSettingHelper.GetAuthServerUri() + "api/userinfo",
                Scope = new List<string> { "openid", "all" },
                Provider = new GenericOAuth2AuthenticationProvider
                {
                    InitRequestFromAuthenticationProperties = EndpointUriTypes => true,
                    OnMapContextToClaims = async context =>
                    {
                        List<Claim> additionalClaims = new List<Claim>();
                        if (!string.IsNullOrEmpty(context.UserProfile.Id))
                        {
                            additionalClaims.Add(new Claim(CelloClaimTypes.UserId, context.UserProfile.Id));
                        }
                        if (!string.IsNullOrEmpty(context.UserProfile.ClientId))
                        {
                            additionalClaims.Add(new Claim(CelloClaimTypes.ClientId, context.UserProfile.ClientId));
                        }
                        if (!string.IsNullOrEmpty(context.UserProfile.TenantId))
                        {
                            additionalClaims.Add(new Claim(CelloClaimTypes.TenantId, context.TenantId));
                            additionalClaims.Add(new Claim(CelloClaimTypes.LoggedInTenantId, context.UserProfile.TenantId));
                        }
                        if (!string.IsNullOrEmpty(context.UserProfile.Scope))
                        {
                            additionalClaims.Add(new Claim(CelloClaimTypes.Scopes, context.UserProfile.Scope));
                        }
                        if (!string.IsNullOrEmpty(context.UserProfile.ProfileUrl))
                        {
                            additionalClaims.Add(new Claim(CelloClaimTypes.Profile, context.UserProfile.ProfileUrl));
                        }

                        if (!string.IsNullOrEmpty(context.UserProfile.Role))
                        {
                            additionalClaims.Add(new Claim(ClaimTypes.Role, string.Join(",", context.UserProfile.Role)));
                            additionalClaims.Add(new Claim(CelloClaimTypes.LoggedInUserRoles, string.Join(",", context.UserProfile.Role)));
                        }
                        if (context.ExpiresIn != null && context.ExpiresIn.HasValue && context.ExpiresIn.Value.TotalSeconds > 0)
                        {
                            additionalClaims.Add(new Claim(ClaimTypes.Expiration, context.ExpiresIn.Value.TotalSeconds.ToString()));
                        }
                        return await Task.FromResult(additionalClaims);
                    }
                }
            };

            app.Map(new PathString("/Account/LogOn"), loginApp =>
            {
                loginApp.UseGenericOAuth2Authentication(authOptions);
            });
        }
    }
}