﻿$(function () {

    disableAlerts();
    $("#actions").hide();

    $("#ClaimMapType").select2({ width: "100%" });

    $("#lstTenants").on("change", function () {

        var tid = $("#lstTenants").val();

        if (tid == "" || tid == null || tid == undefined) {
            ShowClaimMap(false);
            flipContainers();
            return;
        }

        ShowClaimMap();
        loadSettings(tid);
    });

    fillTenants();

    $("#SaveADSetting").click(function () {
        var authorizationSettings = {
            AuthorizationEndPointUri: $("#AuthorizationEndPointUri").val(),
            TokenEndpointUri: $("#TokenEndpointUri").val(),
            UserInfoEndpointUri: $("#UserInfoEndpointUri").val(),
            FedMetadataUrl: $("#FedMetadataUrl").val()
        };

        var setting = {
            AuthorizationSettings: authorizationSettings,
            ClaimMapType: $("#ClaimMapType").val(),
            TenantId: $("#lstTenants").val(),
            ClientId: $("#ClientId").val(),
            ClientSecret: $("#ClientSecret").val()
        };

        $.ajax({
            url: "/AuthSettings/SaveSettings",
            method: "POST",
            contentType: "application/json",
            data: JSON.stringify(setting),
            success: function (result) {

                if (result.Status == "Error") {
                    alert(result.Message, 'Error');
                    return;
                }

                if (result.Status == "Warning") {
                    alert(result.Message, 'Warning');
                    return;
                }

                flipContainers();
                alert(result.Message + " for " + $("#lstTenants :selected").text(), "Success");
                $("#lstTenants option[value='']").prop("selected", true);
                $('#lstTenants').select2({ width: "100%" });
            }
        });
    });

    $("#ResetSetting").click(function () {
        disableAlerts();
        $("#lstTenants option[value='']").prop("selected", true);
        $('#lstTenants').select2({ width: "100%" });
        flipContainers();
    });
});

function flipContainers() {
    $("#ClientId").val('');
    $("#ClientSecret").val('');
    $("#claimMapconfig").hide();
    $("#clientparams").hide();
    $("#endpointconfig").hide();
}

function loadSettings(tenantId) {
    $.ajax({
        url: "/AuthSettings/GetSettings",
        type: "POST",
        data: { tenantId: tenantId },
        success: function (result) {
            if (result == undefined || result == null) {
                $("#endpointconfig").hide();
                alert('No Settings Found', 'Error');
                return;
            }
            $("#actions").show();

            if (result.Status == "Error") {
                $("#endpointconfig").hide();
                alert(result.Message, 'Warning');
                return;
            }

            $('#ClaimMapType option[value="' + result.Data.ClaimMapType + '"]').prop("selected", true);
            $('#ClaimMapType').select2({ width: "100%" });

            var clientParams = result.Data;

            if (clientParams != undefined && clientParams.ClientId != undefined)
                $("#ClientId").val(clientParams.ClientId);
            else
                $("#ClientId").val('');

            if (clientParams != undefined && clientParams.ClientSecret != undefined)
                $("#ClientSecret").val(clientParams.ClientSecret);
            else
                $("#ClientSecret").val('');

            var endpoints = result.Data.AuthorizationSettings;

            if (endpoints != null || endpoints != undefined) {
                $("#AuthorizationEndPointUri").val(endpoints.AuthorizationEndpointUri);
                $("#TokenEndpointUri").val(endpoints.TokenEndpointUri);
                $("#UserInfoEndpointUri").val(endpoints.UserInfoEndpointUri);

                $("#FedMetadataUrl").val(endpoints.FedMetadataUrl);

                if (endpoints.RequireFedMetadata) {
                    $("#fedMetadataContainer").show();
                }
                else {
                    $("#fedMetadataContainer").hide();
                }
                $("#endpointconfig").show();
            }
            else {
                $("#fedMetadataContainer").hide();
                $("#endpointconfig").hide();
            }

            disableAlerts();
        },
        error: function (request, status, error) {
            console.log(request.responseText);
            alerts(request.responseText, "Error");
        }
    });
}

function ShowClaimMap(status) {

    if (status == undefined) {
        $("#claimMapconfig").show();
        $("#clientparams").show();
    }
    else {
        $("#claimMapconfig").hide();
        $("#clientparams").hide();
    }
}

function fillTenants() {
    $.ajax({
        url: "/AuthSettings/GetChildTenants",
        type: "POST",
        success: function (result) {

            if (result.Status != "Success") {
                alert(result.Message, "Warning");
                $('#lstTenants').select2({ width: "100%" });
                return;
            }

            var tenantsLst = result.Data;

            for (var i = 0; i < tenantsLst.length; i++) {
                var tenant = tenantsLst[i];
                $("#lstTenants").append(getOption(tenant.TenantName, tenant.Key));
            }
            $('#lstTenants').select2({ width: "100%" });
        },
        error: function (request, status, error) {
            console.log(request.responseText);
            alert(request.responseText, "Error");
        }
    });
}

function getOption(text, value) {
    var option = $('<option/>').attr({ 'value': value }).text(text);
    return option;
}

/* Custom Alerts definitions */

function bootstrapSuccess(message) {
    var alertDiv = $("<div>").prop("class", "alert alert-success");
    $(alertDiv).append($("<strong>").html("Success!&nbsp;&nbsp;"));
    $(alertDiv).append($("<span>").append($("<span>").html(message)));
    $("#customAlerts").html('').append(alertDiv);
}

function bootstrapErrors(message, isError) {
    var alertDiv = $("<div>").prop("class", "alert alert-warning");
    $(alertDiv).append($("<strong>").html("Warning!&nbsp;&nbsp;"));

    if (isError) {
        alertDiv = $("<div>").prop("class", "alert alert-danger");
        $(alertDiv).append($("<strong>").html("Error!&nbsp;&nbsp;"));
    }

    $(alertDiv).append($("<span>").append($("<span>").html(message)));
    $("#customAlerts").html('').append(alertDiv);
}

function alert(message, alertType) {

    if (alertType == undefined)
        alertType = "Warning";

    if (alertType == "Error") {
        bootstrapErrors(message, true);
    } else if (alertType == "Warning") {
        bootstrapErrors(message, false);
    } else {
        bootstrapSuccess(message);
    }

    $("#customAlerts").show();
}

function disableAlerts() {
    $("#customAlerts").hide();
}