﻿using CelloSaaS.Library;
using CelloSaaS.Model;
using CelloSaaS.Model.ViewManagement;
using CelloSaaS.ServiceContracts.DataManagement;
using CelloSaaS.ServiceProxies.VirtualEntityManagement;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Web;
using System.Web.Mvc;

namespace WebApplication.api
{
    public class ExtendedDataViewController : Controller
    {
        // GET: DataViewRender
        /// <summary>
        /// RenderForm using webcontroller instead of api controller
        /// </summary>
        /// <param name="entity">entity data in json</param>
        /// <param name="name">DataView form name</param>
        /// <param name="formMode">Form Mode</param>
        /// <param name="noOfCols">number of columns</param>
        /// <param name="modelPrefix">model prefix</param>
        /// <param name="autoRender"></param>
        /// <param name="refId">Entity Reference ID (if View or Edit)</param>
        /// <param name="entityId">Entity Identifier(if View or Edit)</param>
        /// <returns></returns>
        public ActionResult RenderForm(string entity, string name, CurrentFormMode formMode, int noOfCols, string modelPrefix, bool autoRender = true, string refId = null, string entityId = null)
        {
           
            entity = HttpUtility.UrlDecode(entity);
            entity = HttpUtility.HtmlDecode(entity);
            var columnAttributes = new Dictionary<int, CelloSaaS.View.ColumnProperties>();
            for (int i = 1; i <= noOfCols; i++)
            {
                columnAttributes.Add(i, new CelloSaaS.View.ColumnProperties());
            }

            Dictionary<string, string> formAttributes = new Dictionary<string, string>();
            formAttributes.Add("id", "frmAdd");
            formAttributes.Add("style", "");
            CelloSaaS.View.AlternateRowStyle alternateRowStyle = new CelloSaaS.View.AlternateRowStyle();
            var jsonBaseEntity = new JsonBaseEntity();
            jsonBaseEntity.JsonEntity = JObject.Parse(entity);

            if (!string.IsNullOrEmpty(refId) && !string.IsNullOrEmpty(entityId))
            {
                IDataManagementService service = ServiceLocator.Resolve<IDataManagementService>();
                var customData = service.GetExtendedEntityRow(entityId, UserIdentity.TenantID.ToString(), refId);
                jsonBaseEntity.ExtendedRow = customData;
            }
            var htmlHelper = new HtmlHelper(new ViewContext(ControllerContext, new WebFormView(ControllerContext, "DataViewRender"), new ViewDataDictionary(), new TempDataDictionary(), TextWriter.Null), new ViewPage());
            htmlHelper.EnableUnobtrusiveJavaScript(false);
            htmlHelper.EnableClientValidation(false);

            var form = new CelloSaaS.View.DataViewTable(name, formMode, formAttributes, htmlHelper, 2, columnAttributes, alternateRowStyle, jsonBaseEntity, modelPrefix, autoRender);

            var htmlValue = form.Render();

            ViewBag.Result = htmlValue;
            return PartialView();
        }
    }
}